package de.intevation.flys.artifacts.model.fixings;

import org.apache.log4j.Logger;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.flys.utils.KMIndex;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.FixationArtifactAccess;

import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.DataFacet;
import de.intevation.flys.artifacts.model.CalculationResult;

import de.intevation.flys.artifacts.states.DefaultState.ComputeType;


/**
 * Facet to show the outliers in a fix calculation.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixOutlierFacet
extends      DataFacet
implements   FacetTypes {

    /** House logger. */
    private static Logger logger = Logger.getLogger(FixOutlierFacet.class);

    /** Trivial Constructor. */
    public FixOutlierFacet() {
    }


    /**
     * @param name
     */
    public FixOutlierFacet(String description) {
        super(0, FIX_OUTLIER, description, ComputeType.ADVANCE, null, null);
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext (ignored).
     *
     * @return the data.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("FixOutlierFacet.getData");

        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact)artifact;
            FixationArtifactAccess access = new FixationArtifactAccess(flys);

            CalculationResult res =
                (CalculationResult) flys.compute(context,
                                                 ComputeType.ADVANCE,
                                                 false);

            FixResult result = (FixResult) res.getData();

            double km = access.getCurrentKm();
            KMIndex<QW []> kmQWs = result.getOutliers();

            QW[] qws = kmQWs.binarySearch(km).getValue();
            if (qws == null) {
                return null;
            }
            return qws;
        }
        else {
            logger.debug("Not an instance of FixationArtifact.");
            return null;
        }
    }


    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public FixOutlierFacet deepCopy() {
        FixOutlierFacet copy = new FixOutlierFacet();
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
